{% extends 'base.html' %}

{% block title %}Register{% endblock %}

{% block content %}
    <h2>You have registered!</h2>
    <p>Click <a href="/accounts/login/">here</a> to login.</p>

{% endblock %}