{% extends 'base.html' %}

{% block title %}About{% endblock %}


{% block content %}
    <div class="container">
        <h2>About</h2>
        <br>

        <div class="panel panel-info">
            <div class="panel-body">
                This example application was developed by <a href="http://www.linkedin.com/in/gadioren/">Gadi Oren</a> as a part of
                moma-django package, in order to demonstrate how the framework can by utilized in a real-life application.
                <p>
                It was deployed and used "live" the first time at the Django Boston meetup 2/27/2014.
                </p>
                <br>
                Please visit the project pages at
                <a href="https://github.com/gadio/moma-django" target="_blank">github</a>.
            </div>
        </div>

        <a href="/q/home/" class="btn btn-default">Back</a>
        </div>
{% endblock %}
