<!DOCTYPE html>
{##==========================================================================#}
{## Copyright 2012 Lucidel, Inc., 2013 Cloudoscope Inc.#}
{###}
{## Licensed under the Apache License, Version 2.0 (the "License");#}
{## you may not use this work except in compliance with the License.#}
{## You may obtain a copy of the License in the LICENSE file, or at:#}
{###}
{## http://www.apache.org/licenses/LICENSE-2.0#}
{###}
{## Unless required by applicable law or agreed to in writing, software#}
{## distributed under the License is distributed on an "AS IS" BASIS,#}
{## WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.#}
{## See the License for the specific language governing permissions and#}
{## limitations under the License.#}
{##==========================================================================#}
{##}
<html lang="en">
{% load staticfiles  %}
{% load i18n %}
{% load url from future %}

<head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{% block title %}{% trans "empty" %}{% endblock %}</title>
    <link rel="shortcut icon" href="/media/favicon.ico" />

    {% block bs_css %}
        <!-- Bootstrap -->
        <link href="/media/bootstrap/css/bootstrap.min.css" rel="stylesheet">
    {% endblock %}
    {% block css %}{% endblock %}
    {% block js %}
        <!-- HTML5 Shim and Respond.js IE8 support of HTML5 elements and media queries -->
        <!-- WARNING: Respond.js doesn't work if you view the page via file:// -->
        <!--[if lt IE 9]>
          <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
          <script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
        <![endif]-->
    {% endblock %}
</head>

<body>
{% block _navbar %}
    {% include "_navbar.html" %}
{% endblock _navbar %}
{% block extra %}{% endblock %}
{% block body %}{% endblock %}
{% block bs_js %}
    <!-- jQuery (necessary for Bootstrap's JavaScript plugins) -->
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/2.1.0/jquery.min.js"></script>
    <!-- Include all compiled plugins (below), or include individual files as needed -->
    <script src="/media/bootstrap/js/bootstrap.min.js"></script>
{% endblock %}
{% block extra_js %}{% endblock %}
</body>
</html>