{% extends 'base.html' %}

{% block title %}Invalid login{% endblock %}

{% block content %}
    <div class="container">
        <h2>Your login details are invalid</h2>
        <p>Click <a href="/accounts/login/">here</a> to login.</p>
    </div>
{% endblock %}