{% extends 'base.html' %}

{% block title %}Logged in{% endblock %}

{% block content %}
    <div class="container">
        <h2>{{ full_name }}, you are logged in!</h2>
        <p>Click <a href="/accounts/logout/">here</a> to logout.</p>
    </div>
{% endblock %}