<fieldset {% if form.prefix %}class="{{ form.prefix }}"{% endif %}>
    <div class="{% if form.prefix %}{{ form.prefix }}-{% endif %}__all__"></div>
    {% if form.non_field_errors %}
        <div class="alert alert-error">
            <a class="close" data-dismiss="alert">&times;</a></a>
            {% for non_field_error in form.non_field_errors %}
                 {{ non_field_error }}
            {% endfor %}
        </div>
    {% endif %}

    {% for field in form.hidden_fields %}
        {{ field }}
    {% endfor %}

    {% for field in form.visible_fields %}
        {% include 'bootstrapform/field.html' %}
    {% endfor %}
</fieldset>

