{% extends 'base_empty.html' %}

{% load staticfiles  %}
{% load i18n %}
{% load url from future %}


{% block body %}
    <div class="container" style="padding: 25px 10px 15px 10px;">
        <div id="main" class="span10"  style="padding: 30px 0px 0px 0px;">
        {% block content %}{% endblock %}
        </div>
    </div>
{% endblock %}
