{% extends 'base.html' %}
{% load bootstrap %}

{% block title %}Register{% endblock %}

{% block content %}
    <h2>Register</h2>

    {% if form.errors %}
        <p class="errors">Not a valid user or password</p>
    {% endif %}

    <form action="/accounts/register/" method="post">{% csrf_token %}
    {{ form|bootstrap }}

    <input type="submit" value="Register">

    </form>

{% endblock %}